package com.fjhx.sale.service.purchase.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.area.utils.CustomizeAreaUtil;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.common.entity.corporation.po.Corporation;
import com.fjhx.common.service.corporation.CorporationService;
import com.fjhx.flow.service.flow.FlowExampleService;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.sale.entity.contract.po.ContractProduct;
import com.fjhx.sale.entity.pack.po.PackDetail;
import com.fjhx.sale.entity.pack.po.PackDetailProduct;
import com.fjhx.sale.entity.pack.vo.PackDetailProductVo;
import com.fjhx.sale.entity.purchase.dto.EhsdPurchaseDto;
import com.fjhx.sale.entity.purchase.dto.EhsdPurchaseSelectDto;
import com.fjhx.sale.entity.purchase.po.*;
import com.fjhx.sale.entity.purchase.vo.EhsdPurchaseProductVo;
import com.fjhx.sale.entity.purchase.vo.EhsdPurchaseVo;
import com.fjhx.sale.entity.sale.vo.SaleQuotationVo;
import com.fjhx.sale.mapper.purchase.EhsdPurchaseMapper;
import com.fjhx.sale.service.pack.PackDetailProductService;
import com.fjhx.sale.service.pack.PackDetailService;
import com.fjhx.sale.service.purchase.*;
import com.fjhx.supply.entity.supplier.po.SupplierInfo;
import com.fjhx.supply.service.supplier.SupplierInfoService;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * ehsd采购表 服务实现类
 * </p>
 *
 * @author
 * @since 2023-05-17
 */
@Service
@DS(SourceConstant.SALE)
public class EhsdPurchaseServiceImpl extends ServiceImpl<EhsdPurchaseMapper, EhsdPurchase> implements EhsdPurchaseService {

    @Autowired
    private EhsdPurchaseProductService ehsdPurchaseProductService;
    @Autowired
    private PackDetailProductService packDetailProductService;
    @Autowired
    private PackDetailService packDetailService;
    @Autowired
    private SupplierInfoService supplierInfoService;
    @Autowired
    private CorporationService corporationService;
    @Autowired
    private FlowExampleService flowExampleService;
    @Autowired
    private EhsdPurchaseProjectService ehsdPurchaseProjectService;
    @Autowired
    private EhsdPurchaseArrivalService ehsdPurchaseArrivalService;
    @Autowired
    private ProductInfoService productInfoService;
    @Autowired
    private EhsdPurchaseProductMountingsService ehsdPurchaseProductMountingsService;

    /**
     * 分页
     *
     * @param dto
     * @return
     */
    @Override
    public Page<EhsdPurchaseVo> getPage(EhsdPurchaseSelectDto dto) {
        IWrapper<EhsdPurchase> wrapper = getWrapper();
        wrapper.orderByDesc("t1", EhsdPurchase::getCreateTime);
        if (ObjectUtils.isNotEmpty(dto.getPurchaseStatus())) {
            wrapper.eq("t1", EhsdPurchase::getStatus, dto.getPurchaseStatus());
        }
        if (ObjectUtils.isNotEmpty(dto.getPayStatus())) {
            wrapper.eq("t1", EhsdPurchase::getPayStatus, dto.getPayStatus());
        }
        if (ObjectUtils.isNotEmpty(dto.getKeyword())) {
            wrapper.keyword(dto.getKeyword(), new SqlField("t2.name"), new SqlField("t1.`code`"));
        }
        Page<EhsdPurchaseVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<EhsdPurchaseVo> records = page.getRecords();
        //  赋值流程id
        flowExampleService.setFlowId(records, EhsdPurchaseVo::setFlowId);

        return page;
    }

    @Override
    public EhsdPurchaseVo detail(Long id) {
        EhsdPurchase ehsdPurchase = this.getById(id);
        EhsdPurchaseVo result = BeanUtil.toBean(ehsdPurchase, EhsdPurchaseVo.class);

        List<Long> ids = new ArrayList<>();
//        ids.add(result.getBuyCountryId());
//        ids.add(result.getBuyProvinceId());
//        ids.add(result.getBuyCityId());
        ids.add(result.getSellCountryId());
        ids.add(result.getSellProvinceId());
        ids.add(result.getSellCityId());
        Map<Long, String> areaMapByIds = CustomizeAreaUtil.getAreaMapByIds(ids);
        //赋值卖方地址
        result.setSellCountryName(areaMapByIds.get(result.getSellCountryId()));
        result.setSellProvinceName(areaMapByIds.get(result.getSellProvinceId()));
        result.setSellCityName(areaMapByIds.get(result.getSellCityId()));
        //赋值买方地址
//        result.setBuyCountryName(areaMapByIds.get(result.getBuyCountryId()));
//        result.setBuyProvinceName(areaMapByIds.get(result.getBuyProvinceId()));
//        result.setBuyCityName(areaMapByIds.get(result.getBuyCityId()));

        //赋值卖方名称
        SupplierInfo supplierInfo = supplierInfoService.getById(result.getSellCorporationId());
        if (ObjectUtils.isNotEmpty(supplierInfo)) {
            result.setSellCorporationName(supplierInfo.getName());
        }
        //赋值卖方（供应商）银行信息
        result.setOpeningBank(supplierInfo.getOpeningBank());//开户银行
        result.setOpeningName(supplierInfo.getOpeningName());//账户名
        result.setContactPerson(supplierInfo.getContactPerson());//联系人
        result.setAccountOpening(supplierInfo.getAccountOpening());//账户
        //赋值买方名称
        Corporation corporation = corporationService.getById(result.getBuyCorporationId());
        if (ObjectUtils.isNotEmpty(corporation)) {
            result.setBuyCorporationName(corporation.getName());
        }
        //赋值采购明细
        List<EhsdPurchaseProduct> ehsdPurchaseProductList = ehsdPurchaseProductService.list(q ->
                q.eq(EhsdPurchaseProduct::getPurchaseId, ehsdPurchase.getId())
        );
        List<EhsdPurchaseProductVo> ehsdPurchaseProductVos = BeanUtil.copyToList(ehsdPurchaseProductList, EhsdPurchaseProductVo.class);
        //赋值产品配件信息
        if (ObjectUtils.isNotEmpty(ehsdPurchaseProductList)) {
            List<Long> eppIds = ehsdPurchaseProductVos.stream().map(EhsdPurchaseProductVo::getId).collect(Collectors.toList());
            Map<Long, List<EhsdPurchaseProductMountings>> eppMap = ehsdPurchaseProductMountingsService.mapKGroup(
                    EhsdPurchaseProductMountings::getPurchaseProductId, q -> q
                    .in(EhsdPurchaseProductMountings::getPurchaseProductId, eppIds)
            );
            for (EhsdPurchaseProductVo ehsdPurchaseProductVo : ehsdPurchaseProductVos) {
                List<EhsdPurchaseProductMountings> ehsdPurchaseProductMountings = eppMap.get(ehsdPurchaseProductVo.getId());
                ehsdPurchaseProductVo.setPurchaseProductMountingsList(ehsdPurchaseProductMountings);
            }
        }

        //赋值产品信息
        productInfoService.attributeAssign(ehsdPurchaseProductVos, EhsdPurchaseProductVo::getProductId, (item, product) -> {
            item.setDefinition(product.getDefinition());
        });
        result.setEhsdPurchaseProductList(ehsdPurchaseProductVos);
        //赋值出货明细
        List<Long> cpIds = ehsdPurchaseProductVos.stream().map(EhsdPurchaseProductVo::getDataResourceId).collect(Collectors.toList());
        List<PackDetailProduct> packDetailProductList = packDetailProductService.list(q ->
                q.in(PackDetailProduct::getContractProductId, cpIds)
        );
        List<PackDetailProductVo> packDetailProductVos = BeanUtil.copyToList(packDetailProductList, PackDetailProductVo.class);
        //赋值出货时间
        if (ObjectUtils.isNotEmpty(packDetailProductVos)) {
            PackDetail packDetail = packDetailService.getById(packDetailProductVos.get(0).getPackDetailId());
            packDetailProductVos.forEach(item -> item.setShipmentTime(packDetail.getShipmentTime()));
        }
        result.setPackDetailProductList(packDetailProductVos);


        //赋值其他收费项目
        List<EhsdPurchaseProject> ehsdPurchaseProjects = ehsdPurchaseProjectService.list(q -> q.eq(EhsdPurchaseProject::getPurchaseId, id));
        result.setPurchaseProjectList(ehsdPurchaseProjects);
        //赋值到货信息
        List<EhsdPurchaseArrival> ehsdPurchaseArrivalList = ehsdPurchaseArrivalService.list(q -> q.eq(EhsdPurchaseArrival::getPurchaseId, id));
        productInfoService.attributeAssign(ehsdPurchaseArrivalList, EhsdPurchaseArrival::getProductId, (item, product) -> {
            item.setProductName(product.getName());
            item.setProductCode(product.getCode());
        });
        result.setPurchaseArrivalList(ehsdPurchaseArrivalList);

        return result;
    }

    @Override
    public void add(EhsdPurchaseDto ehsdPurchaseDto) {
        this.save(ehsdPurchaseDto);
    }

    @Override
    public void edit(EhsdPurchaseDto ehsdPurchaseDto) {
        this.updateById(ehsdPurchaseDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

}
